import cv2

def preprocess_frame(frame):
    frame_resized = cv2.resize(frame, (224, 224))
    frame_bgr2rgb = cv2.cvtColor(frame_resized, cv2.COLOR_BGR2RGB)
    return frame_bgr2rgb

def preprocess_frames(frames):
    preprocessed_frames = []
    for frame in frames:
        preprocessed_frame = preprocess_frame(frame)
        preprocessed_frames.append(preprocessed_frame)
    return preprocessed_frames



from transformers import AutoImageProcessor, VideoMAEImageProcessor, VideoMAEForVideoClassification

# other models => "MCG-NJU/videomae-base" 
def load_model(label2id, id2label, model_name="MCG-NJU/videomae-base-finetuned-kinetics"):
    processor = AutoImageProcessor.from_pretrained(model_name)
    print(label2id)
    model = VideoMAEForVideoClassification.from_pretrained(model_name, 
                                                           label2id=label2id,
                                                            id2label=id2label,
                                                            ignore_mismatched_sizes=True)
    return model, processor


import sys
import torch
from torch.utils.data import DataLoader
from sklearn.model_selection import train_test_split

from threat_scanner.training.train import train
from threat_scanner.training.evaluate import evaluate
from threat_scanner.training.load_model import load_model
from threat_scanner.training.load_threat import load_threat
from threat_scanner.training.save_model import save_model
from threat_scanner.training.dataset import ThreatDataset

def main():
    device = "cuda" if torch.cuda.is_available() else "cpu"
    num_epochs = 10
    learning_rate = 1e-4
    batch_size = 4
    model_path = "finetuned_videomae_threats.pth"

    threats = load_threat()
    if not threats:
        print("No data specified for training")
        sys.exit(1)

    directories = [threat.path for threat in threats]
    label2id = {threat.name: threat.id for _, threat in enumerate(threats)}
    id2label = {threat.id: threat.name for _, threat in enumerate(threats)}

    model, processor = load_model(label2id, id2label)
    
    labels = [i for i in range(len(directories))]
    dataset = ThreatDataset(directories, labels, processor)

    train_data, test_data = train_test_split(dataset.data, test_size=0.2, random_state=42)
    train_data, val_data = train_test_split(train_data, test_size=0.2, random_state=42)

    train_dataset = ThreatDataset(directories, labels, processor)
    train_dataset.data = train_data

    val_dataset = ThreatDataset(directories, labels, processor)
    val_dataset.data = val_data

    test_dataset = ThreatDataset(directories, labels, processor)
    test_dataset.data = test_data
    

    train_loader = DataLoader(train_dataset, batch_size=batch_size, shuffle=True)
    val_loader = DataLoader(val_dataset, batch_size=batch_size, shuffle=False)
    test_loader = DataLoader(test_dataset, batch_size=batch_size, shuffle=False)

    criterion = torch.nn.CrossEntropyLoss()
    optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate)

    train(model, train_loader, val_loader, optimizer, criterion, device, num_epochs=num_epochs)
    evaluate(model, test_loader, device)
    save_model(model, model_path)
    print(f"Model saved to {model_path} ")

if __name__ == "__main__":
    main()


import os 
import sys
import argparse

from threat_scanner.detector.threat import ThreatDetector

def main():
    parser = argparse.ArgumentParser(description="Discover threats from video files or live feed.")
    parser.add_argument(
        "--source",
        "-s",
        help="Path to the video file. If not provided, defaults to live feed.",
        default=0,
        dest="source", #store in args.source
    )
    parser.add_argument(
        "--model",
        "-m",
        help="Model to classify video. If not provided, defaults to videomae-base-finetuned-kinetics",
        default="MCG-NJU/videomae-base-finetuned-kinetics",
        dest="model", #store in args.model
    )

    args = parser.parse_args()
    source = args.source
    model = args.model

    if source != 0 and not os.path.exists(source):
        print(f"Error: Video file not found at path: {source}")
        sys.exit(1)
    
    detector = ThreatDetector(model)
    detector.scan(source=source)

if __name__ == "__main__":
    main()


import cv2
import torch
from transformers import VideoMAEImageProcessor, VideoMAEForVideoClassification
from threat_scanner.tools.frame_preprocess import preprocess_frame

class ThreatDetector:
    def __init__(self, model_name):
        self.model_name = model_name
        self.device = "cuda" if torch.cuda.is_available() else "cpu"
        self.processor = VideoMAEImageProcessor.from_pretrained(self.model_name)
        self.model = VideoMAEForVideoClassification.from_pretrained(self.model_name)
        self.model.to(self.device)
        self.num_frames_to_predict = 17
        self.frame_diff_threshold = 0.1
    
    def predict(self, frames):
        if not frames:
            return None
        inputs = self.processor(list(frames), return_tensors="pt")
        inputs = {k: v.to(self.device) for k, v in inputs.items()}
        with torch.no_grad():
            outputs = self.model(**inputs)
            logits = outputs.logits
        probs = torch.nn.functional.softmax(logits, dim=-1)
        predicted_class_idx = torch.argmax(probs, dim=-1).item()
        predicted_class_name = self.model.config.id2label[predicted_class_idx]
        predicted_class_prob = probs[0][predicted_class_idx].item()
        return (predicted_class_name, predicted_class_prob)
    
    def scan(self, source):
        cap = cv2.VideoCapture(source)
        
        if not cap.isOpened():
            print("Error opening video file")
            return
        
        
        frames = []
        previous_frame = None
        prediction = None
        probability = None
        index = 0
        while(cap.isOpened()):
            ret, frame = cap.read()
            if ret:
                processed_frame = preprocess_frame(frame.copy())
                if previous_frame is not None:
                    diff = cv2.absdiff(previous_frame, processed_frame)
                    mean_diff = diff.mean()
                    if mean_diff > self.frame_diff_threshold:
                        frames.append(processed_frame)
                        if len(frames) == self.num_frames_to_predict:
                            prediction, probability = self.predict(frames)
                            frames = []
                else:
                    frames.append(processed_frame)
                    if len(frames) == self.num_frames_to_predict:
                        prediction, probability = self.predict(frames)
                        frames = []
                if prediction and probability:
                    cv2.putText(frame, f"{prediction} {probability}", (50, 70), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 255), 4)
                cv2.imshow('Frame', frame)

                # Wait for 25 milliseconds for a key press. If 'q' is pressed, exit the loop
                if cv2.waitKey(5) & 0xFF == ord('q'):
                    break
            else:
                break
            previous_frame = processed_frame
            index += 1
        print("Done")
        cap.release()
        cv2.destroyAllWindows()




class Threat:
    def __init__(self, data):
        self.name = None
        self.id = None
        self.path = None

        if "_id" in data:
            self.id = data["_id"]
        else:
            raise ValueError("Threat must have an _id")
        
        if "name" in data:
            self.name = data["name"]
        else:
            raise ValueError("Threat must have a name")
        
        if "path" in data:
            self.path = data["path"]
        else:
            raise ValueError("Threat must have a path")



import torch
from tqdm import tqdm

def train(model, train_loader, val_loader, optimizer, criterion, device, num_epochs=10):
    model.to(device)
    for epoch in range(num_epochs):
        model.train()
        total_loss = 0
        print(f"\n--- Starting Epoch {epoch+1}/{num_epochs} ---") # Epoch start marker
        for i, batch_data in enumerate(tqdm(train_loader, desc=f"Epoch {epoch+1}/{num_epochs} (Training)")):
            inputs_dict = batch_data[0]
            labels = batch_data[1].to(device)

            # 1. Get the pixel_values tensor from the batch
            pixel_values_from_loader = inputs_dict['pixel_values'].to(device)

            # 2. Attempt to squeeze the tensor
            pixel_values_for_model = pixel_values_from_loader # Initialize with the original

            if pixel_values_from_loader.ndim == 6 and pixel_values_from_loader.shape[1] == 1:
                pixel_values_for_model = pixel_values_from_loader.squeeze(1)

            # 3. Model forward pass
            optimizer.zero_grad()
            outputs = model(pixel_values=pixel_values_for_model)
            logits = outputs.logits
            
            # 4. Loss calculation and backward pass
            loss = criterion(logits, labels)
            loss.backward()
            optimizer.step()
            total_loss += loss.item()

        avg_train_loss = total_loss / len(train_loader)
        print(f"Epoch {epoch+1}/{num_epochs}, Average Training Loss: {avg_train_loss:.4f}")

        # Validation loop (apply similar debugging if issues persist there)
        model.eval()
        total_val_loss = 0
        with torch.no_grad():
            for i_val, batch_data_val in enumerate(tqdm(val_loader, desc=f"Epoch {epoch+1}/{num_epochs} (Validation)")):
                inputs_dict_val = batch_data_val[0]
                labels_val = batch_data_val[1].to(device)

                pixel_values_from_loader_val = inputs_dict_val['pixel_values'].to(device)
                pixel_values_for_model_val = pixel_values_from_loader_val

                if pixel_values_from_loader_val.ndim == 6 and pixel_values_from_loader_val.shape[1] == 1:
                    pixel_values_for_model_val = pixel_values_from_loader_val.squeeze(1)

                outputs_val = model(pixel_values=pixel_values_for_model_val)
                logits_val = outputs_val.logits
                loss_val = criterion(logits_val, labels_val)
                total_val_loss += loss_val.item()

        avg_val_loss = total_val_loss / len(val_loader) if len(val_loader) > 0 else 0
        print(f"Epoch {epoch+1}/{num_epochs}, Average Validation Loss: {avg_val_loss:.4f}")



import os
import cv2
import torch
import numpy as np
from tqdm import tqdm
from torch.utils.data import Dataset
from threat_scanner.tools.frame_preprocess import preprocess_frames

class ThreatDataset(Dataset):
    def __init__(self, directories, labels, processor, max_frames=16):
        self.directories = directories
        self.labels = labels
        self.processor = processor
        self.max_frames = max_frames
        self.data = []
        self._load_data()

    def _load_data(self):
        for directory, label in zip(self.directories, self.labels):
          for filename in os.listdir(directory):
            if filename.endswith(".mov") or filename.endswith(".avi") or filename.endswith(".mp4"):
                video_path = os.path.join(directory, filename)
                self.data.append((video_path, label))

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        video_path, label = self.data[idx]
        cap = cv2.VideoCapture(video_path)
        frames = []
        while True:
            ret, frame = cap.read()
            if not ret:
                break
            frames.append(frame)
        cap.release()

        if len(frames) > self.max_frames:
            start_index = np.random.randint(0, len(frames) - self.max_frames)
            frames = frames[start_index:start_index + self.max_frames]

        frames = preprocess_frames(frames)
        inputs = self.processor(list(frames), return_tensors="pt")
        return inputs, torch.tensor(label)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="threat_scanner", 
    version="0.1.2",         
    author="Ogo-oluwasubomi Popoola",
    author_email="popoolaogooluwasubomi@gmail.com",
    description="ThreatScan is an open-source Python package designed to detect potential physical threats in videos",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/threatscan-ai/threatscan",
    packages=find_packages(), 
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
)



import torch
from tqdm import tqdm

def evaluate(model, test_loader, device):
    model.to(device)
    model.eval()
    correct = 0
    total = 0
    with torch.no_grad():
        for batch_data in tqdm(test_loader, desc="Evaluating"):
            inputs_dict = batch_data[0]
            labels = batch_data[1].to(device)

            pixel_values = inputs_dict['pixel_values'].to(device)
            if pixel_values.ndim == 6 and pixel_values.shape[1] == 1:
                pixel_values = pixel_values.squeeze(1)

            outputs = model(pixel_values=pixel_values) # Pass pixel_values directly
            logits = outputs.logits
            _, predicted = torch.max(logits, 1)
            total += labels.size(0)
            correct += (predicted == labels).sum().item()

    accuracy = 100 * correct / total
    print(f"Test Accuracy: {accuracy:.2f}%")


import json

from threat_scanner.training.threat import Threat

def load_threat(filepath="threat_scanner/training/threats.json"):
    try:
        with open(filepath, 'r') as file:
            data = json.load(file)
            threats = []
            for threat_data in list(data.values()):
                threats.append(Threat(threat_data))
            print("Threats successfully loaded!")
            return threats
    except FileNotFoundError:
        print(f"Error: File not found: {filepath}")
        return None
    except json.JSONDecodeError:
        print(f"Error: Invalid JSON format in {filepath}")
        return None
    except Exception as e:
         print(f"An unexpected error occurred: {e}")
         return None